// (C) Copyright 1999 by Autodesk, Inc. 
//
// Permission to use, copy, modify, and distribute this software in
// object code form for any purpose and without fee is hereby granted, 
// provided that the above copyright notice appears in all copies and 
// that both that copyright notice and the limited warranty and
// restricted rights notice below appear in all supporting 
// documentation.
//
// AUTODESK PROVIDES THIS PROGRAM "AS IS" AND WITH ALL FAULTS. 
// AUTODESK SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF
// MERCHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK, INC. 
// DOES NOT WARRANT THAT THE OPERATION OF THE PROGRAM WILL BE
// UNINTERRUPTED OR ERROR FREE.
//
// Use, duplication, or disclosure by the U.S. Government is subject to 
// restrictions set forth in FAR 52.227-19 (Commercial Computer
// Software - Restricted Rights) and DFAR 252.227-7013(c)(1)(ii)
// (Rights in Technical Data and Computer Software), as applicable.
//
// Created by: Jorge Lopez
//             ObjectDBX Development
//             ADF / Engineering
//             Autodesk, Inc.

//-----------------------------------------------------------------------------
#include "StdAfx.h"
#include "axdockbar.h"

//-----------------------------------------------------------------------------
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


STDMETHODIMP_(ULONG) CAxContainerBar::AddRef(void)
{
    return ++m_refCount;
}

STDMETHODIMP_(ULONG) CAxContainerBar::Release(void)
{
    if (m_refCount == 1)
    {
        delete this;
        return 0L;
    }
    else
        return --m_refCount;
}

STDMETHODIMP CAxContainerBar::QueryInterface(REFIID riid, LPVOID*ppv) 
{
    *ppv = NULL;

    if (IsEqualIID(riid, IID_IUnknown))
        *ppv = static_cast<IUnknown*>(this);
    else if(IsEqualIID(riid, IID_IDispatch))
        *ppv = static_cast<IDispatch*>(this);
    else if(IsEqualIID(riid, IID_IAcadDockableContainer))
        *ppv = static_cast<IAcadDockableContainer*>(this);
    else
        return E_NOINTERFACE;

    ((LPUNKNOWN) *ppv)->AddRef();
    return S_OK;
}


STDMETHODIMP CAxContainerBar::GetTypeInfoCount(UINT* pctinfo)
{
    *pctinfo = 1;
    return S_OK;
}

STDMETHODIMP CAxContainerBar::GetTypeInfo(UINT itinfo, LCID lcid, ITypeInfo** pptinfo)
{
    if (itinfo != 0) 
        return TYPE_E_ELEMENTNOTFOUND;
    if (NULL == pptinfo) 
        return E_POINTER;

    *pptinfo = NULL;
    return AfxGetApp()->GetTypeInfoOfGuid(0x0, IID_IAcadDockableContainer, pptinfo);
}

STDMETHODIMP CAxContainerBar::GetIDsOfNames(REFIID riid, OLECHAR** rgszNames, 
                                            UINT cNames, LCID lcid, DISPID* rgdispid)
{
    if (riid != IID_NULL) 
        return DISP_E_UNKNOWNINTERFACE;

    LPTYPEINFO pTypeInfo = NULL;
    HRESULT hr = GetTypeInfo(0, lcid, &pTypeInfo);
    if (SUCCEEDED(hr))
    {
        hr = pTypeInfo->GetIDsOfNames(rgszNames, cNames, rgdispid);
        pTypeInfo->Release();
    }

    return hr;
}

STDMETHODIMP CAxContainerBar::Invoke(DISPID dispidMember, REFIID riid, LCID lcid, 
                                     WORD wFlags, DISPPARAMS* pdispparams, 
                                     VARIANT* pvarResult, EXCEPINFO* pexcepinfo, 
                                     UINT* puArgErr)
{
    if (riid != IID_NULL) 
        return DISP_E_UNKNOWNINTERFACE;

    LPTYPEINFO pTypeInfo = NULL;
    HRESULT hr = GetTypeInfo(0, lcid, &pTypeInfo);

    if (SUCCEEDED(hr))
    {
        SetErrorInfo(0L, NULL);

        hr = pTypeInfo->Invoke(static_cast<IDispatch*>(this), dispidMember, 
                               wFlags, pdispparams, pvarResult, pexcepinfo, 
                               puArgErr);
        pTypeInfo->Release();
    }

    return hr;
}

STDMETHODIMP CAxContainerBar::Float(void)
{
    if (!m_containerBar->IsFloating())
    {
        m_containerBar->ToggleDocking();
    }

    return S_OK;
}

STDMETHODIMP CAxContainerBar::Dock(void)
{
    if (m_containerBar->IsFloating())
    {
        m_containerBar->ToggleDocking();
    }

    return S_OK;
}

STDMETHODIMP CAxContainerBar::put_Caption(BSTR newVal)
{
    CString caption(newVal);
    m_containerBar->SetWindowText(caption);

    // update the floating frame as well
    if (m_containerBar->IsFloating())
    {
        CFrameWnd* pFrameWnd = m_containerBar->GetParentFrame();
        ASSERT(pFrameWnd != NULL);

        pFrameWnd->SetWindowText(caption);

        // CMiniDockFrameWnd will resync the caption using the
        // dockbar's caption, therefore, to prevent the caption
        // from changing again during a resize or move event it
        // is required to set the window text in the dock bar as well.
        // So in effect, we have to tell the system 3 times what we 
        // want. :-(
        CWnd* pParent = m_containerBar->GetParent();
        pParent->SetWindowText(caption);
    }

    return S_OK;
}

STDMETHODIMP CAxContainerBar::get_Caption(BSTR* pVal)
{
    CString caption;
    m_containerBar->GetWindowText(caption);

    *pVal = caption.AllocSysString();   // come back and catch possible exception
    return S_OK;
}

STDMETHODIMP CAxContainerBar::put_AllowDocking(VARIANT_BOOL newVal)
{
    if (((m_containerBar->m_dwDockStyle & CBRS_ALIGN_ANY) != 0) != 
         (newVal == VARIANT_TRUE))
    {
        m_containerBar->ToggleAllowDocking();
    }
    
    return S_OK;
}

STDMETHODIMP CAxContainerBar::get_AllowDocking(VARIANT_BOOL* pVal)
{
    if ((m_containerBar->m_dwDockStyle & CBRS_ALIGN_ANY) != 0)
        *pVal = VARIANT_TRUE;
    else
        *pVal = VARIANT_FALSE;
    
    return S_OK;
}

STDMETHODIMP CAxContainerBar::get_IsFloating(VARIANT_BOOL* pVal)
{
    *pVal = m_containerBar->IsFloating() ? VARIANT_TRUE : VARIANT_FALSE;
    return S_OK;
}

STDMETHODIMP CAxContainerBar::put_EnableDockPositions(long newVal)
{
    // must be CBRS_ALIGN_XXX or CBRS_FLOAT_MULTI only
    if ((newVal & ~(CBRS_ALIGN_ANY|CBRS_FLOAT_MULTI)) != 0)
        return E_INVALIDARG;
    
    m_containerBar->EnableDocking(newVal);
    m_containerBar->SetEnableDockingCalled(true);
    return S_OK;
}

STDMETHODIMP CAxContainerBar::get_EnableDockPositions(long* pVal)
{
    *pVal = m_containerBar->m_dwDockStyle & CBRS_ALIGN_ANY;    
    return S_OK;
}

STDMETHODIMP CAxContainerBar::SetPreferredDockPosition(long DockPosition, VARIANT FloatingRect)
{
	// MFC has different values for specifying sides when calling the frame's
	// DockControlBar() function. I will hide this from the exposed Automation
	// API to keep it simple. This function will simply do the conversion.
	UINT dockPosition;
	switch(DockPosition)
	{
	case acFloating:
		dockPosition = DockPosition;
		break;
	case acDockLeft:
		dockPosition = AFX_IDW_DOCKBAR_LEFT;
		break;
	case acDockTop:
		dockPosition = AFX_IDW_DOCKBAR_TOP;
		break;
	case acDockRight:
		dockPosition = AFX_IDW_DOCKBAR_RIGHT;
		break;
	case acDockBottom:
		dockPosition = AFX_IDW_DOCKBAR_BOTTOM;
		break;
	default:
		return E_INVALIDARG;
	}

    m_containerBar->SetPreferredDockPosition(dockPosition);

    // Handle optional floating rect size, if and only if a floating
    // window is preferred.
    if (DockPosition == acFloating)
    {
        if (!(V_VT(&FloatingRect) == VT_ERROR ||
              V_ERROR(&FloatingRect) == DISP_E_PARAMNOTFOUND))
        {
            VARIANT floatingRect;
            VariantInit(&floatingRect);

            try
            {
                // Copy the VARIANT to rid ourselves of the posibility that
                // it was passed in BYREF; then check to ensure the VARIANT
                // contains the information we need.
                HRESULT hr = VariantCopyInd(&floatingRect, &FloatingRect);
                if (FAILED(hr))
                    throw hr;
                else 
                if (!(V_ISARRAY(&floatingRect) &&
                    ((VT_TYPEMASK & V_VT(&floatingRect)) == VT_I2 ||
                     (VT_TYPEMASK & V_VT(&floatingRect)) == VT_I4)))
                {
                    throw E_INVALIDARG;
                }

                // Get the rect values from the SAFEARRAY.
                SAFEARRAY* saRect = V_ARRAY(&floatingRect);
                if (saRect == NULL || SafeArrayGetDim(saRect) != 1)
                {
                    throw E_INVALIDARG;
                }

                long LowerBound;
                long UpperBound;

                if (FAILED(hr = SafeArrayGetLBound(saRect, 1, &LowerBound)))
                    throw hr;
                if (FAILED(hr = SafeArrayGetUBound(saRect, 1, &UpperBound)))
                    throw hr;

                // The user is required to specify all four corners of the
                // rectangle.
                if ((UpperBound - LowerBound + 1) != 4)
                    throw E_INVALIDARG;

                CRect rect(0, 0, 0, 0); // Initialize incase VT_I2 where used.
                if (FAILED(hr = SafeArrayGetElement(saRect, &LowerBound, (LPVOID) &rect.left)))
                    throw hr;
                LowerBound++;

                if (FAILED(hr = SafeArrayGetElement(saRect, &LowerBound, (LPVOID) &rect.top)))
                    throw hr;
                LowerBound++;

                if (FAILED(hr = SafeArrayGetElement(saRect, &LowerBound, (LPVOID) &rect.right)))
                    throw hr;
                LowerBound++;
                
                if (FAILED(hr = SafeArrayGetElement(saRect, &LowerBound, (LPVOID) &rect.bottom)))
                    throw hr;

                VariantClear(&floatingRect);

                // All that work so I can do this. :-(
                m_containerBar->SetPreferredFloatingRect(rect);
            }
            catch(HRESULT hr)
            {
                VariantClear(&floatingRect);
                return hr;
            }
        }
    }

    return S_OK;
}
